package ch.unisg.executorpool;

import static org.mockito.Mockito.times;

import java.net.URI;
import java.util.Optional;

import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.BDDMockito.then;

import ch.unisg.executorpool.application.port.in.AddNewExecutorToExecutorPoolCommand;
import ch.unisg.executorpool.application.port.out.AddExecutorPort;
import ch.unisg.executorpool.application.port.out.ExecutorAddedEventPort;
import ch.unisg.executorpool.application.service.AddNewExecutorToExecutorPoolService;
import ch.unisg.executorpool.domain.ExecutorClass;
import ch.unisg.executorpool.domain.ExecutorPool;

public class AddNewExecutorToExecutorPoolServiceTest {

    private final AddExecutorPort addExecutorPort = Mockito.mock(AddExecutorPort.class);
    private final ExecutorAddedEventPort newExecutorAddedEventPort = Mockito.mock(ExecutorAddedEventPort.class);
    private final AddNewExecutorToExecutorPoolService addNewExecutorToExecutorPoolService = new AddNewExecutorToExecutorPoolService(
        newExecutorAddedEventPort, addExecutorPort); 

    @Test
    void addingSucceeds() {

        ExecutorClass newExecutor = givenAnExecutorWithTypeAndUri(new ExecutorClass.ExecutorTaskType("test-type"), 
        Optional.of(new ExecutorClass.ExecutorUri(URI.create("example.org"))));

        ExecutorPool executorPool = givenAnEmptyExecutorPool(ExecutorPool.getExecutorPool());

        AddNewExecutorToExecutorPoolCommand addNewExecutorToExecutorPoolCommand = new AddNewExecutorToExecutorPoolCommand(newExecutor.getExecutorTaskType(),
        Optional.ofNullable(newExecutor.getExecutorUri()));

        ExecutorClass addedExecutor = addNewExecutorToExecutorPoolService.addNewExecutorToExecutorPool(addNewExecutorToExecutorPoolCommand);

        assertThat(addedExecutor).isNotNull();
        assertThat(executorPool.getListOfExecutors().getValue()).hasSize(1);

        then(newExecutorAddedEventPort).should(times(1)).publishNewExecutorAddedEvent(any(NewExecutorAddedEvent.class));

    }


    private ExecutorPool givenAnEmptyExecutorPool(ExecutorPool executorPool) {
        executorPool.getListOfExecutors().getValue().clear();
        return executorPool;
    } 

    private ExecutorClass givenAnExecutorWithTypeAndUri(ExecutorClass.ExecutorTaskType executorTaskType, 
    Optional<ExecutorClass.ExecutorUri> executorUri) {

        ExecutorClass executor = Mockito.mock(ExecutorClass.class);
        given(ExecutorClass.getExecutorTaskType()).willReturn(executorTaskType);
        given(ExecutorClass.getExecutorUri()).willReturn(executorUri);
        return executor;
    }


    
}
